"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importStar(require("react"));
const react_alert_1 = require("react-alert");
const formik_1 = require("formik");
const Yup = __importStar(require("yup"));
const contexts_1 = require("../../contexts");
const objects_1 = require("@hypernetlabs/objects");
const components_1 = require("../../components");
var ERegistryAction;
(function (ERegistryAction) {
    ERegistryAction[ERegistryAction["ADD_REGISTRY"] = 0] = "ADD_REGISTRY";
    ERegistryAction[ERegistryAction["ADD_GATEWAY"] = 1] = "ADD_GATEWAY";
})(ERegistryAction || (ERegistryAction = {}));
const gatewaysRegistryName = "Gateways";
const CreateProposalWidget = ({ onProposalListNavigate, }) => {
    const alert = (0, react_alert_1.useAlert)();
    const { coreProxy, UIData } = (0, contexts_1.useStoreContext)();
    const { setLoading } = (0, contexts_1.useLayoutContext)();
    const [accountAddress, setAccountAddress] = (0, react_1.useState)((0, objects_1.EthereumAddress)(""));
    const [proposalThreshold, setProposalThreshold] = (0, react_1.useState)();
    const [votingPower, setVotingPower] = (0, react_1.useState)();
    const [gatewayRegistry, setGatewayRegistry] = (0, react_1.useState)();
    (0, react_1.useEffect)(() => {
        coreProxy
            .getProposalThreshold()
            .map((_proposalThreshold) => {
            setProposalThreshold(_proposalThreshold);
        })
            .mapErr(handleError);
        coreProxy
            .getRegistryByName([gatewaysRegistryName])
            .map((registryMap) => {
            setGatewayRegistry(registryMap.get(gatewaysRegistryName));
        })
            .mapErr(handleError);
        coreProxy
            .getEthereumAccounts()
            .map((accounts) => {
            setAccountAddress(accounts[0]);
            coreProxy
                .getVotingPower(accounts[0])
                .map((power) => {
                setVotingPower(power);
            })
                .mapErr(handleError);
        })
            .mapErr(handleError);
    }, []);
    const handleError = (err) => {
        setLoading(false);
        alert.error((err === null || err === void 0 ? void 0 : err.message) || "Something went wrong!");
    };
    const handleCreatePrposalFormSubmit = async (values, formikHelpers) => {
        setLoading(true);
        if (values.action == ERegistryAction.ADD_REGISTRY) {
            coreProxy
                .createProposal(values.name, values.symbol, (0, objects_1.EthereumAddress)(values.owner))
                .map((proposal) => {
                setLoading(false);
                onProposalListNavigate && onProposalListNavigate();
            })
                .mapErr(handleError);
        }
        else {
            coreProxy
                .proposeRegistryEntry(gatewaysRegistryName, values.gatewayUrl, JSON.stringify({
                gatewayName: values.gatewayName,
                gatewayLogoUrl: values.gatewayLogoUrl,
                gatewayAddress: values.gatewayAddress,
                gatewaySignature: values.gatewaySignature,
            }), (0, objects_1.EthereumAddress)(values.owner))
                .map((proposal) => {
                setLoading(false);
                onProposalListNavigate && onProposalListNavigate();
            })
                .mapErr(handleError);
        }
    };
    const isAccountGatewayRegistrarAddress = (0, react_1.useMemo)(() => {
        return gatewayRegistry === null || gatewayRegistry === void 0 ? void 0 : gatewayRegistry.registrarAddresses.includes(accountAddress);
    }, [gatewayRegistry, accountAddress]);
    const actionList = (0, react_1.useMemo)(() => {
        const actions = [
            {
                primaryText: "Add Registry",
                secondaryText: undefined,
                action: null,
                value: ERegistryAction.ADD_REGISTRY,
            },
        ];
        if (isAccountGatewayRegistrarAddress) {
            actions.push({
                primaryText: "Add Gateway",
                secondaryText: undefined,
                action: null,
                value: ERegistryAction.ADD_GATEWAY,
            });
        }
        return actions;
    }, [gatewayRegistry]);
    const exceedsThreshold = (0, react_1.useMemo)(() => {
        return !votingPower || !proposalThreshold
            ? false
            : votingPower >= proposalThreshold;
    }, [proposalThreshold, votingPower]);
    return (react_1.default.createElement(react_1.default.Fragment, null,
        react_1.default.createElement(components_1.GovernanceWidgetHeader, { label: "Create Proposal", navigationLink: {
                label: "Proposal list",
                onClick: () => {
                    onProposalListNavigate && onProposalListNavigate();
                },
            } }),
        react_1.default.createElement(formik_1.Formik, { initialValues: {
                action: ERegistryAction.ADD_REGISTRY,
                name: "",
                symbol: "",
                gatewayUrl: "",
                gatewayAddress: "",
                gatewaySignature: "",
                gatewayName: "",
                gatewayLogoUrl: "",
                owner: accountAddress,
            }, enableReinitialize: true, onSubmit: handleCreatePrposalFormSubmit, validationSchema: Yup.object().shape({
                action: Yup.number().required("Required"),
            }) }, ({ handleSubmit, isSubmitting, values }) => (react_1.default.createElement(formik_1.Form, { onSubmit: handleSubmit, id: "CreateProposalForm" },
            react_1.default.createElement(components_1.GovernanceDialogSelectLargeField, { name: "action", title: "Proposed Action", type: "select", placeholder: "select action", required: true, options: actionList }),
            values.action == ERegistryAction.ADD_REGISTRY && (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(components_1.GovernanceLargeField, { title: "Name", name: "name", type: "input", placeholder: "Type a Name", required: values.action == ERegistryAction.ADD_REGISTRY }),
                react_1.default.createElement(components_1.GovernanceLargeField, { title: "Symbol", name: "symbol", type: "input", placeholder: "Type a Symbol", required: values.action == ERegistryAction.ADD_REGISTRY }),
                react_1.default.createElement(components_1.GovernanceLargeField, { title: "Owner", name: "owner", type: "input", placeholder: "Type an Owner", required: values.action == ERegistryAction.ADD_REGISTRY }))),
            values.action == ERegistryAction.ADD_GATEWAY && (react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(components_1.GovernanceLargeField, { title: "Gateway URL", name: "gatewayUrl", type: "input", placeholder: "Type a Gateway URL", required: values.action == ERegistryAction.ADD_GATEWAY }),
                react_1.default.createElement(components_1.GovernanceLargeField, { title: "Address", name: "gatewayAddress", type: "input", placeholder: "Type gateway address", required: false }),
                react_1.default.createElement(components_1.GovernanceLargeField, { title: "Signature", name: "gatewaySignature", type: "input", placeholder: "Type gateway Signature", required: false }),
                react_1.default.createElement(components_1.GovernanceLargeField, { title: "Name", name: "gatewayName", type: "input", placeholder: "Type gateway Name", required: false }),
                react_1.default.createElement(components_1.GovernanceLargeField, { title: "Logo URL", name: "gatewayLogoUrl", type: "input", placeholder: "Type gateway logo URL", required: false }))),
            react_1.default.createElement(components_1.GovernanceButton, { variant: "contained", size: "large", form: "CreateProposalForm", type: "submit", fullWidth: true, color: "primary", disabled: !exceedsThreshold, onClick: () => { } }, exceedsThreshold
                ? "Submit"
                : `You must have ${proposalThreshold} tokens to submit a proposal`))))));
};
exports.default = CreateProposalWidget;
//# sourceMappingURL=CreateProposalWidget.js.map